@@ -23,5 +23,6 @@ void make_dialogs(WXwidget parent){
   appdata.delete_local=create_delete_dialog(parent,LOCAL);*/
   appdata.delete_dialog=create_delete_dialog(parent);
   appdata.xfer_dialog=create_xfer_dialog(parent);
-  
+  appdata.progress=create_progress_dialog(parent);
+  appdata.progress_shown=0; 
 }
