@@ -105,6 +105,9 @@ void update_dirlist(GtkWidget *table,dirinfo* di){
   GdkBitmap* bit;
   int max[NUMFIELDS];
   GtkStyle *style;
+  gchar* dummy[NUMFIELDS];
+
+  for(i=0;i<NUMFIELDS;i++)dummy[i]="";
 
   style=gtk_widget_get_style(table);
 
@@ -113,7 +116,7 @@ void update_dirlist(GtkWidget *table,dirinfo* di){
 
   max[0]=3;for(i=1;i<NUMFIELDS;i++)max[i]=6;
   for(i=0;i<di->total;i++){
-    gtk_clist_append(GTK_CLIST(table),NULL);
+    gtk_clist_append(GTK_CLIST(table),dummy);
     for(j=0;j<NUMFIELDS;j++){
 
       switch(j){
@@ -335,20 +338,47 @@ static int motion_cb(GtkWidget* w,GdkEventMotion* e,gpointer app){
 GtkWidget* create_dirlist(GtkWidget* parent,int which){
   struct _select *select_struct;
   GtkWidget* list;
+#ifdef NEW_GTK
+  GtkWidget* sw;
+#endif
   static gchar* titles[NUMFIELDS]={"^","Name","Date","Time","Size"};
   static GtkJustification justs[NUMFIELDS+1]={GTK_JUSTIFY_CENTER,
     GTK_JUSTIFY_LEFT,GTK_JUSTIFY_CENTER,GTK_JUSTIFY_CENTER,GTK_JUSTIFY_RIGHT};
   int i,w,t;
 
+#ifdef NEW_GTK
+  sw=gtk_scrolled_window_new(NULL,NULL);
+  gtk_container_set_border_width(GTK_CONTAINER(sw),0);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+   GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+#endif
+
   list=gtk_clist_new_with_titles(NUMFIELDS,titles);
-  /*gtk_clist_set_selection_mode(GTK_CLIST(list),GTK_SELECTION_EXTENDED);*/
+
+#ifdef NEW_GTK
+  gtk_container_add(GTK_CONTAINER(sw),list);
+#endif
+
   gtk_clist_set_selection_mode(GTK_CLIST(list),-1);
-  gtk_clist_set_policy(GTK_CLIST(list),
-      GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+
+  /*gtk_clist_set_policy(GTK_CLIST(list),
+      GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);*/
+
+#ifdef NEW_GTK
+  gtk_clist_set_shadow_type(GTK_CLIST(list),GTK_SHADOW_IN);
+#elif defined OLD_GTK
   gtk_clist_set_border(GTK_CLIST(list),GTK_SHADOW_IN);
+#else
+#error Unknown gtk+ version!
+#endif
+
   gtk_clist_set_row_height(GTK_CLIST(list),gtkfontheight*1.6);
 
+#ifdef OLD_GTK
   GTK_CLIST_CLASS(GTK_OBJECT(list)->klass)->scrollbar_spacing=0;
+#endif
+
+
 
   w=0;
   for(i=0;i<NUMFIELDS;i++){
@@ -385,7 +415,15 @@ GtkWidget* create_dirlist(GtkWidget* parent,int which){
       break;
   }
 
+#ifdef NEW_GTK
+  gtk_widget_show(list);
+  gtk_widget_show(sw);
+  return sw;
+#elif defined OLD_GTK
   return list;
+#else
+#error Unknown gtk+ version
+#endif
 }
 
 #elif defined BUILD_MOTIF
@@ -434,8 +472,10 @@ void doubleclick_cb(Widget w,XtPointer app,XtPointer call){
       appdata.job=0;
       break;
   }
-  /*printf("DOUBLECLICK for %s row #%d\n",(which==REMOTE)?"remote":"local",
-      cbs->row);*/
+  /*
+  printf("DOUBLECLICK for %s row #%d\n",(which==REMOTE)?"remote":"local",
+      cbs->row);
+      */
   return;
 }
 
@@ -610,10 +650,10 @@ Widget create_dirlist(Widget parent,int which){
   XColor color,exact;
   static String labels[NUMFIELDS]={"^","Name","Date","Time","Size"};
   static char translations[]=
-   "<Btn1Up>:	DefaultAction() SelectCell(Up) \n\
+   "<Btn1Up>:           	SelectCell(Up) DefaultAction() \n\
     ~Shift Ctrl <Btn1Down>:	SelectCell(Toggle) \n\
     Shift ~Ctrl <Btn1Down>:	SelectCell(Extend) \n\
-    ~Shift ~Ctrl <Btn1Down>:	DefaultAction() SelectCell(Select) \n";
+    ~Shift ~Ctrl <Btn1Down>:	SelectCell(Select) DefaultAction() \n";
   /*
   static char translations[]=
    "<Btn1Up>:	DefaultAction() SelectCell(Up) \n\
