@@ -1,4 +1,4 @@
-/* Copyright (c) 1998   Alexander Yukhimets. All rights reserved. */
+/* Copyright (c) 1998,1999   Alexander Yukhimets. All rights reserved. */
 #include<stdio.h>
 #include<string.h>
 #include<stdlib.h>
@@ -26,7 +26,11 @@
 #include"functions.h"
 
 int show_help(int num){
+#ifdef WXFTP_HELP_DIR
+  static char* help_loc=WXFTP_HELP_DIR "index.html";
+#else
   static char* help_loc="/usr/local/share/wxftp/help/index.html";
+#endif /*WXFTP_HELP_DIR*/
   char* buf;
   buf=WXmalloc(strlen(help_loc)*3+80);
   sprintf(buf,
@@ -61,6 +65,10 @@ static int update_file_progress(int len,void* arg){
   
   currenttime=time(NULL);
   newsize=data->currentsize+(long)len;
+  if(currenttime==data->lasttime){
+    data->currentsize=newsize;
+    return check_for_small_interrupt();
+  }
   newelapsed=(currenttime-data->lasttime);
   newelapsed+=data->elapsed;
   if(newelapsed)newrate=((float)newsize)/newelapsed/1024;
@@ -83,7 +91,7 @@ static int update_file_progress(int len,void* arg){
   if(newpercent==data->percent)newpercent=-1;else data->percent=newpercent;
   update_progress_dialog(data->progress,newpercent,newsize,newrate,
       newelapsed,newleft);
-  return check_for_interrupt();
+  return check_for_small_interrupt();
 }
 
 int check_for_interrupt(){
@@ -220,7 +228,6 @@ int view_remote_num(int num){
   FILE* tmp;
   struct _file_progress_data fpd;
   WXwidget viewer;
-  WXwidget progress;
 
   if(appdata.remote.list->files[num+1]->perms[0]!='-'){
     (void)popup_warning_dialog(toplevel,
@@ -244,13 +251,14 @@ int view_remote_num(int num){
   if(ret)return 2;
   if(appdata.connect.lastline[0]!='1')return 3;
 
-  progress=create_progress_dialog(toplevel);
-  init_progress_dialog(progress,appdata.remote.list->files[num+1]->name,
+/*  progress=create_progress_dialog(toplevel);*/
+  init_progress_dialog(appdata.progress,
+                appdata.remote.list->files[num+1]->name,
                 appdata.remote.list->files[num+1]->size);
 
   process_events();
 
-  fpd.progress=progress;
+  fpd.progress=appdata.progress;
   fpd.totalsize=appdata.remote.list->files[num+1]->size;
   fpd.lasttime=time(NULL);
   fpd.currentsize=0;
@@ -261,7 +269,7 @@ int view_remote_num(int num){
 
   ret=ftp_xfer_get('A',appdata.connect.dfd,fileno(tmp),update_file_progress,
       (void*)&fpd);
-  hide_progress_dialog(progress);
+  hide_progress_dialog(appdata.progress);
   if(appdata.odata->xferbeep)beep();
   if(ret==-1){
     append_status("CANCELLED\n");
@@ -363,12 +371,20 @@ int delete_remote(char* name){
   return ret;
 }
 
+int check_for_small_interrupt(){
+  process_events();
+  if(appdata.interrupt || appdata.small_interrupt){
+    appdata.small_interrupt=0;
+    return 1;
+  }
+  return 0;
+}
+
 int upload_file(fileinfo* fi,char type){
   int ret;
   int lfd;
   struct _file_progress_data fpd;
   struct stat statbuf;
-  volatile WXwidget progress;
   int show_progress;
 
   if((lfd=open(fi->name,O_RDONLY))<0){
@@ -388,7 +404,7 @@ int upload_file(fileinfo* fi,char type){
     return 2;
   }
   if(!type)type='I';
-  ret=ftp_put(type,fi->name,&appdata.connect,logfile,check_for_interrupt);
+  ret=ftp_put(type,fi->name,&appdata.connect,logfile,check_for_small_interrupt);
   if(ret==10){
     /*
     append_status("CONNECTION LOST\n");
@@ -416,12 +432,16 @@ int upload_file(fileinfo* fi,char type){
     if(tres<0)tres=0;
     if(statbuf.st_size/1024>=tres){
 
-      progress=create_progress_dialog(toplevel);
-      init_progress_dialog(progress,fi->name,statbuf.st_size);
+      /*progress=create_progress_dialog(toplevel);*/
+      if(!appdata.progress_shown){
+	show_progress_dialog(appdata.progress);
+	appdata.progress_shown=1;
+      }
+      init_progress_dialog(appdata.progress,fi->name,statbuf.st_size);
 
       process_events();
 
-      fpd.progress=progress;
+      fpd.progress=appdata.progress;
       fpd.totalsize=statbuf.st_size;
       fpd.lasttime=time(NULL);
       fpd.currentsize=0;
@@ -441,21 +461,21 @@ int upload_file(fileinfo* fi,char type){
 
   close(lfd);
   if(show_progress){
-    hide_progress_dialog(progress);
+    init_progress_dialog(appdata.progress,"",0);
   }
   if(appdata.odata->xferbeep)beep();
   if(ret==-1){
     append_status("CANCELLED\n");
-    if(!ftp_abort_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_abort_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   } else if(ret){
     append_status("CONNECTION FAILED\n");
-    if(!ftp_close_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_close_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   } else {
-    if(!ftp_close_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_close_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   }
@@ -467,13 +487,13 @@ int download_file(fileinfo* volatile fi,char volatile type){
   volatile int lfd;
   volatile struct _file_progress_data fpd;
   struct stat statbuf;
-  volatile WXwidget progress;
   long size;
   int show_progress;
 
   if(!type)type='I';
 
-  ftp_size(&size,type,fi->name,&appdata.connect,logfile,check_for_interrupt);
+  ftp_size(&size,type,fi->name,&appdata.connect,logfile,
+      check_for_small_interrupt);
   if(size<=0){
     size=fi->size;
   }
@@ -499,7 +519,8 @@ int download_file(fileinfo* volatile fi,char volatile type){
       WXfree(b);
       return 2;
     }
-    ret=ftp_get(type,fi->name,&appdata.connect,logfile,check_for_interrupt);
+    ret=ftp_get(type,fi->name,&appdata.connect,logfile,
+	check_for_small_interrupt);
     fpd.totalsize=size;
   } else if(ret==2){
     if((lfd=open(fi->name,O_WRONLY|O_APPEND,0755))<0){
@@ -511,7 +532,7 @@ int download_file(fileinfo* volatile fi,char volatile type){
       return 2;
     }
     ret=ftp_resume(type,statbuf.st_size,fi->name,&appdata.connect,logfile,
-	check_for_interrupt);
+	check_for_small_interrupt);
     fpd.totalsize=size-statbuf.st_size;
     if(fpd.totalsize<0)fpd.totalsize=0;
   } else if(ret==3){
@@ -533,12 +554,16 @@ int download_file(fileinfo* volatile fi,char volatile type){
     if(tres<0)tres=0;
     if(size/1024>=tres || (fi->perms[0] == 'l' && size==fi->size)){
 
-      progress=create_progress_dialog(toplevel);
-      init_progress_dialog(progress,fi->name,size);
+      /*progress=create_progress_dialog(toplevel);*/
+      if(!appdata.progress_shown){
+	show_progress_dialog(appdata.progress);
+	appdata.progress_shown=1;
+      }
+      init_progress_dialog(appdata.progress,fi->name,size);
 
       process_events();
 
-      fpd.progress=progress;
+      fpd.progress=appdata.progress;
       fpd.currentsize=0;
       fpd.lasttime=time(NULL);
       fpd.elapsed=0;
@@ -558,21 +583,21 @@ int download_file(fileinfo* volatile fi,char volatile type){
 
   close(lfd);
   if(show_progress){
-    hide_progress_dialog(progress);
+    init_progress_dialog(appdata.progress,"",0);
   }
   if(appdata.odata->xferbeep)beep();
   if(ret==-1){
     append_status("CANCELLED\n");
-    if(!ftp_abort_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_abort_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   } else if(ret){
     append_status("CONNECTION FAILED\n");
-    if(!ftp_close_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_close_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   } else {
-    if(!ftp_close_data(&appdata.connect,logfile,check_for_interrupt)){
+    if(!ftp_close_data(&appdata.connect,logfile,check_for_small_interrupt)){
       append_status(appdata.connect.lastline);
     }
   }
