@@ -63,9 +63,12 @@ static void listaction_cb(GtkWidget* w){
 }
 
 static void anon_cb(GtkWidget *w,gpointer x){
-  GtkWidget *passfield,*user;
+  GtkWidget *user;
+  GtkWidget *passfield;
   char *pass;
+#ifdef OLD_GTK
   char* stars;
+#endif
 
   user=(GtkWidget*)x;
   passfield=(GtkWidget*)gtk_object_get_user_data(GTK_OBJECT(user));
@@ -73,21 +76,34 @@ static void anon_cb(GtkWidget *w,gpointer x){
 
   if(GTK_TOGGLE_BUTTON(w)->active){
     gtk_object_set_user_data(GTK_OBJECT(w),(gpointer)TRUE);
+#ifdef NEW_GTK
+    strcpy(pass,gtk_entry_get_text(GTK_ENTRY(passfield)));
+    gtk_entry_set_visibility(GTK_ENTRY(passfield),TRUE);
+#endif
     gtk_entry_set_text(GTK_ENTRY(passfield),appdata.odata->anonpass);
     gtk_entry_set_text(GTK_ENTRY(user),"anonymous");
   } else {
+#ifdef OLD_GTK
     gtk_object_set_user_data(GTK_OBJECT(w),(gpointer)TRUE);
+#endif
     gtk_entry_set_text(GTK_ENTRY(passfield),pass);
+#ifdef NEW_GTK
+    gtk_entry_set_visibility(GTK_ENTRY(passfield),FALSE);
+#elif defined OLD_GTK
     stars=WXmalloc(strlen(pass)+1);
     memset(stars,'*',strlen(pass));
     stars[strlen(pass)]='\0';
     gtk_entry_set_text(GTK_ENTRY(passfield),stars);
     WXfree(stars);
+#else 
+#error Unknown GTK+ version
+#endif
     gtk_object_set_user_data(GTK_OBJECT(w),(gpointer)FALSE);
   }
 
 }
 
+#ifdef OLD_GTK
 static void password_cb(GtkWidget* w,gpointer x){
   GtkWidget *anon;
   char* pass,*text;
@@ -118,6 +134,7 @@ static void password_cb(GtkWidget* w,gpointer x){
 
   return;
 }
+#endif
 
 GtkWidget* create_session_general(GtkWidget *parent){
   static char* label[]={
@@ -180,7 +197,10 @@ GtkWidget* create_session_general(GtkWidget *parent){
   gtk_table_attach_defaults(GTK_TABLE(table),passfield,1,2,3,4);
   gtk_object_set_user_data(GTK_OBJECT(passfield),(gpointer)passbuf);
   gtk_object_set_user_data(GTK_OBJECT(user),(gpointer)passfield);
-
+#ifdef NEW_GTK
+  gtk_entry_set_visibility(GTK_ENTRY(passfield),FALSE);
+#endif
+  
   child=gtk_entry_new();
   gtk_widget_set_usize(child,0,gtkfontheight*2);
   gtk_widget_set_name(child,"account");
@@ -242,8 +262,10 @@ GtkWidget* create_session_general(GtkWidget *parent){
   gtk_widget_show(current);
   gtk_box_pack_start(GTK_BOX(table),current,TRUE,TRUE,4);
 
+#ifdef OLD_GTK
   gtk_signal_connect(GTK_OBJECT(passfield),"changed",
       GTK_SIGNAL_FUNC(password_cb),(gpointer)anontoggle);
+#endif
   gtk_signal_connect(GTK_OBJECT(anontoggle),"toggled",
       GTK_SIGNAL_FUNC(anon_cb),(gpointer)user);
   return general;
